@@ -16,15 +16,15 @@ from types import ModuleType
 from typing import *
 
 from hippolyzer.lib.base.datatypes import UUID
+from hippolyzer.lib.base.message.message import Message
+from hippolyzer.lib.base.network.transport import UDPPacket
 from hippolyzer.lib.proxy import addon_ctx
 from hippolyzer.lib.proxy.task_scheduler import TaskLifeScope, TaskScheduler
 
 if TYPE_CHECKING:
     from hippolyzer.lib.proxy.commands import CommandDetails, WrappedCommandCallable
     from hippolyzer.lib.proxy.http_flow import HippoHTTPFlow
-    from hippolyzer.lib.proxy.message import ProxiedMessage
-    from hippolyzer.lib.proxy.objects import Object
-    from hippolyzer.lib.proxy.packets import ProxiedUDPPacket
+    from hippolyzer.lib.proxy.object_manager import Object
     from hippolyzer.lib.proxy.region import ProxiedRegion
     from hippolyzer.lib.proxy.sessions import Session, SessionManager
 
@@ -55,6 +55,10 @@ class BaseInteractionManager:
     async def save_file(self, caption: str = '', directory: str = '', filter_str: str = '') -> Optional[str]:
         pass
 
+    @abc.abstractmethod
+    async def confirm(self, title: str, caption: str) -> bool:
+        pass
+
     def main_window_handle(self) -> Any:
         return None
 
@@ -97,9 +101,14 @@ class AddonManager:
 
     @classmethod
     def shutdown(cls):
+        to_pop = []
         for mod in cls.FRESH_ADDON_MODULES.values():
+            to_pop.append(mod)
             cls._call_module_hooks(mod, "handle_unload", cls.SESSION_MANAGER)
         cls.SCHEDULER.shutdown()
+        for mod in to_pop:
+            if isinstance(mod, ModuleType):
+                sys.modules.pop(mod.__name__, None)
 
     @classmethod
     def have_active_repl(cls):
@@ -169,6 +178,7 @@ class AddonManager:
         old_mod = cls.FRESH_ADDON_MODULES.pop(specs[0].name, None)
         if old_mod:
             cls._unload_module(old_mod)
+            sys.modules.pop(old_mod.__name__, None)
         if reload:
             cls._reload_addons()
 
@@ -385,7 +395,7 @@ class AddonManager:
         LOG.error(text)
 
     @classmethod
-    def handle_lludp_message(cls, session: Session, region: ProxiedRegion, message: ProxiedMessage):
+    def handle_lludp_message(cls, session: Session, region: ProxiedRegion, message: Message):
         cls._reload_addons()
         if message.name == "ChatFromViewer" and "ChatData" in message:
             if message["ChatData"]["Channel"] == cls.COMMAND_CHANNEL:
@@ -517,8 +527,13 @@ class AddonManager:
             return cls._call_all_addon_hooks("handle_region_changed", session, region)
 
     @classmethod
-    def handle_proxied_packet(cls, session_manager: SessionManager, packet: ProxiedUDPPacket,
-                              session: Optional[Session], region: Optional[ProxiedRegion],
-                              message: Optional[ProxiedMessage]):
-        return cls._call_all_addon_hooks("handle_proxied_packet", session_manager,
-                                         packet, session, region, message)
+    def handle_circuit_created(cls, session: Session, region: ProxiedRegion):
+        with addon_ctx.push(session, region):
+            return cls._call_all_addon_hooks("handle_circuit_created", session, region)
+
+    @classmethod
+    def handle_proxied_packet(cls, session_manager: SessionManager, packet: UDPPacket,
+                              session: Optional[Session], region: Optional[ProxiedRegion]):
+        with addon_ctx.push(session, region):
+            return cls._call_all_addon_hooks("handle_proxied_packet", session_manager,
+                                             packet, session, region)
