@@ -82,6 +82,7 @@ class HTTPFlowContext:
         self.from_proxy_queue = multiprocessing.Queue()
         self.to_proxy_queue = multiprocessing.Queue()
         self.shutdown_signal = multiprocessing.Event()
+        self.mitmproxy_ready = multiprocessing.Event()
 
 
 class IPCInterceptionAddon:
@@ -249,9 +250,9 @@ def create_http_proxy(bind_host, port, flow_context: HTTPFlowContext):  # pragma
 
 def is_asset_server_cap_name(cap_name):
     return cap_name and (
-            cap_name.startswith("GetMesh") or
-            cap_name.startswith("GetTexture") or
-            cap_name.startswith("ViewerAsset")
+        cap_name.startswith("GetMesh")
+        or cap_name.startswith("GetTexture")
+        or cap_name.startswith("ViewerAsset")
     )
 
 
