@@ -5,10 +5,9 @@ from typing import Optional, Tuple
 from hippolyzer.lib.base.message.message_dot_xml import MessageDotXML
 from hippolyzer.lib.base.message.udpdeserializer import UDPMessageDeserializer
 from hippolyzer.lib.base.message.udpserializer import UDPMessageSerializer
-from hippolyzer.lib.base.settings import Settings
 from hippolyzer.lib.proxy.addons import AddonManager
-from hippolyzer.lib.proxy.packets import ProxiedUDPPacket
-from hippolyzer.lib.proxy.message import ProxiedMessage
+from hippolyzer.lib.base.network.transport import UDPPacket
+from hippolyzer.lib.base.message.message import Message
 from hippolyzer.lib.proxy.region import ProxiedRegion
 from hippolyzer.lib.proxy.sessions import Session, SessionManager
 from hippolyzer.lib.proxy.socks_proxy import SOCKS5Server, UDPProxyProtocol
@@ -26,56 +25,39 @@ class SLSOCKS5Server(SOCKS5Server):
         return lambda: InterceptingLLUDPProxyProtocol(source_addr, self.session_manager)
 
 
-class BaseLLUDPProxyProtocol(UDPProxyProtocol):
-    def __init__(self, source_addr: Tuple[str, int]):
+class InterceptingLLUDPProxyProtocol(UDPProxyProtocol):
+    def __init__(self, source_addr: Tuple[str, int], session_manager: SessionManager):
         super().__init__(source_addr)
-        self.settings = Settings()
-        self.settings.ENABLE_DEFERRED_PACKET_PARSING = True
-        self.settings.HANDLE_PACKETS = False
+        self.session_manager: SessionManager = session_manager
         self.serializer = UDPMessageSerializer()
         self.deserializer = UDPMessageDeserializer(
-            settings=self.settings,
-            message_cls=ProxiedMessage,
+            settings=self.session_manager.settings,
         )
         self.message_xml = MessageDotXML()
+        self.session: Optional[Session] = None
 
-    def _ensure_message_allowed(self, msg: ProxiedMessage):
+    def _ensure_message_allowed(self, msg: Message):
         if not self.message_xml.validate_udp_msg(msg.name):
             LOG.warning(
                 f"Received {msg.name!r} over UDP, when it should come over the event queue. Discarding."
             )
             raise PermissionError(f"UDPBanned message {msg.name}")
 
-
-class InterceptingLLUDPProxyProtocol(BaseLLUDPProxyProtocol):
-    def __init__(self, source_addr: Tuple[str, int], session_manager: SessionManager):
-        super().__init__(source_addr)
-        self.session_manager: SessionManager = session_manager
-        self.session: Optional[Session] = None
-
-    def _handle_proxied_packet(self, packet: ProxiedUDPPacket):
-        message: Optional[ProxiedMessage] = None
+    def handle_proxied_packet(self, packet: UDPPacket):
         region: Optional[ProxiedRegion] = None
         # Try to do an initial region lookup so we have it for handle_proxied_packet()
         if self.session:
             region = self.session.region_by_circuit_addr(packet.far_addr)
-        deserialize_exc = None
-        try:
-            message = self.deserializer.deserialize(packet.data)
-            message.direction = packet.direction
-        except Exception as e:
-            # Hang onto this since handle_proxied_packet doesn't need a parseable
-            # message. If that hook doesn't handle the packet then re-raise.
-            deserialize_exc = e
 
+        # the proxied packet handler is allowed to mutate `packet.data` before
+        # the message gets parsed.
         if AddonManager.handle_proxied_packet(self.session_manager, packet,
-                                              self.session, region, message):
-            # Swallow any error raised by above message deserialization, it was handled.
+                                              self.session, region):
             return
 
-        if deserialize_exc is not None:
-            # handle_proxied_packet() didn't deal with the error, so it's fatal.
-            raise deserialize_exc
+        message = self.deserializer.deserialize(packet.data)
+        message.direction = packet.direction
+        message.sender = packet.src_addr
 
         assert message is not None
         # Check for UDP bans on inbound messages
@@ -122,7 +104,19 @@ class InterceptingLLUDPProxyProtocol(BaseLLUDPProxyProtocol):
                 region.handle = message["Data"]["RegionHandle"]
             LOG.info(f"Setting main region to {region!r}, had circuit addr {packet.far_addr!r}")
             AddonManager.handle_region_changed(self.session, region)
+        if message.name == "RegionHandshake":
+            region.cache_id = message["RegionInfo"]["CacheID"]
+            self.session.objects.track_region_objects(region.handle)
+            if self.session_manager.settings.USE_VIEWER_OBJECT_CACHE:
+                try:
+                    region.objects.load_cache()
+                except:
+                    LOG.exception("Failed to load region cache, skipping")
 
+        try:
+            self.session.message_handler.handle(message)
+        except:
+            LOG.exception("Failed in session message handler")
         try:
             region.message_handler.handle(message)
         except:
@@ -134,6 +128,12 @@ class InterceptingLLUDPProxyProtocol(BaseLLUDPProxyProtocol):
             self.session, region, message
         )
 
+        # This message is owned by an async handler, drop it so it doesn't get
+        # sent with the normal flow.
+        if message.queued and not message.dropped:
+            region.circuit.drop_message(message)
+
+        # Shouldn't mutate the message past this point, so log it now.
         if message_logger:
             message_logger.log_lludp_message(self.session, region, message)
 
@@ -145,11 +145,6 @@ class InterceptingLLUDPProxyProtocol(BaseLLUDPProxyProtocol):
         elif message.name == "RegionHandshake":
             region.name = str(message["RegionInfo"][0]["SimName"])
 
-        # This message is owned by an async handler, drop it so it doesn't get
-        # sent with the normal flow.
-        if message.queued and not message.dropped:
-            region.circuit.drop_message(message)
-
         if not message.dropped:
             region.circuit.send_message(message)
 
