@@ -15,12 +15,14 @@ from defusedxml import minidom
 from hippolyzer.lib.base import serialization as se, llsd
 from hippolyzer.lib.base.datatypes import TaggedUnion, UUID, TupleCoord
 from hippolyzer.lib.base.helpers import bytes_escape
-from hippolyzer.lib.proxy.message_filter import MetaFieldSpecifier, compile_filter, BaseFilterNode, MessageFilterNode
+from hippolyzer.lib.base.message.message_formatting import HumanMessageSerializer
+from hippolyzer.lib.proxy.message_filter import MetaFieldSpecifier, compile_filter, BaseFilterNode, MessageFilterNode, \
+    EnumFieldSpecifier
 from hippolyzer.lib.proxy.region import CapType
 
 if typing.TYPE_CHECKING:
     from hippolyzer.lib.proxy.http_flow import HippoHTTPFlow
-    from hippolyzer.lib.proxy.message import ProxiedMessage
+    from hippolyzer.lib.base.message.message import Message
     from hippolyzer.lib.proxy.region import ProxiedRegion
     from hippolyzer.lib.proxy.sessions import Session
 
@@ -28,7 +30,7 @@ LOG = logging.getLogger(__name__)
 
 
 class BaseMessageLogger:
-    def log_lludp_message(self, session: Session, region: ProxiedRegion, message: ProxiedMessage):
+    def log_lludp_message(self, session: Session, region: ProxiedRegion, message: Message):
         pass
 
     def log_http_response(self, flow: HippoHTTPFlow):
@@ -61,7 +63,7 @@ class FilteringMessageLogger(BaseMessageLogger):
     def set_paused(self, paused: bool):
         self._paused = paused
 
-    def log_lludp_message(self, session: Session, region: ProxiedRegion, message: ProxiedMessage):
+    def log_lludp_message(self, session: Session, region: ProxiedRegion, message: Message):
         if self._paused:
             return
         self._add_log_entry(LLUDPMessageLogEntry(message, region, session))
@@ -254,6 +256,11 @@ class AbstractMessageLogEntry:
                     expected = expected()
                 else:
                     expected = str(expected)
+        elif isinstance(expected, EnumFieldSpecifier):
+            # Local import so we get a fresh copy of the templates module
+            from hippolyzer.lib.proxy import templates
+            enum_cls = getattr(templates, expected.enum_name)
+            expected = enum_cls[expected.field_name]
         elif expected is not None:
             # Unbox the expected value
             expected = expected.value
@@ -286,6 +293,8 @@ class AbstractMessageLogEntry:
             return val > expected
         elif operator == ">=":
             return val >= expected
+        elif operator == "&":
+            return val & expected
         else:
             raise ValueError(f"Unexpected operator {operator!r}")
 
@@ -359,8 +368,8 @@ class HTTPMessageLogEntry(AbstractMessageLogEntry):
         cap_name = cap_data and cap_data.cap_name
         base_url = cap_name and cap_data.base_url
         temporary_cap = cap_data and cap_data.type == CapType.TEMPORARY
-        beautify_url = (beautify and base_url and cap_name and
-                        not temporary_cap and self.session and want_request)
+        beautify_url = (beautify and base_url and cap_name
+                        and not temporary_cap and self.session and want_request)
         if want_request:
             buf.write(message.method)
             buf.write(" ")
@@ -487,7 +496,7 @@ class EQMessageLogEntry(AbstractMessageLogEntry):
         return "EQ"
 
     def request(self, beautify=False, replacements=None):
-        return self._format_llsd(self.event["body"])
+        return f'EQ {self.event["message"]}\n\n{self._format_llsd(self.event["body"])}'
 
     @property
     def name(self):
@@ -505,8 +514,8 @@ class EQMessageLogEntry(AbstractMessageLogEntry):
 class LLUDPMessageLogEntry(AbstractMessageLogEntry):
     __slots__ = ["_message", "_name", "_direction", "_frozen_message", "_seq", "_deserializer"]
 
-    def __init__(self, message: ProxiedMessage, region, session):
-        self._message: ProxiedMessage = message
+    def __init__(self, message: Message, region, session):
+        self._message: Message = message
         self._deserializer = None
         self._name = message.name
         self._direction = message.direction
@@ -531,7 +540,7 @@ class LLUDPMessageLogEntry(AbstractMessageLogEntry):
         return super()._get_meta(name)
 
     @property
-    def message(self):
+    def message(self) -> Message:
         if self._message:
             return self._message
         elif self._frozen_message:
@@ -542,12 +551,16 @@ class LLUDPMessageLogEntry(AbstractMessageLogEntry):
             raise ValueError("Didn't have a fresh or frozen message somehow")
 
     def freeze(self):
-        self.message.invalidate_caches()
+        message = self.message
+        message.invalidate_caches()
         # These are expensive to keep around. pickle them and un-pickle on
         # an as-needed basis.
         self._deserializer = self.message.deserializer
-        self.message.deserializer = None
-        self._frozen_message = pickle.dumps(self._message, protocol=pickle.HIGHEST_PROTOCOL)
+        message.deserializer = None
+        try:
+            self._frozen_message = pickle.dumps(self._message, protocol=pickle.HIGHEST_PROTOCOL)
+        finally:
+            message.deserializer = self._deserializer
         self._message = None
 
     @property
@@ -567,7 +580,7 @@ class LLUDPMessageLogEntry(AbstractMessageLogEntry):
         return self._direction.name if self._direction is not None else ""
 
     def request(self, beautify=False, replacements=None):
-        return self.message.to_human_string(replacements, beautify)
+        return HumanMessageSerializer.to_human_string(self.message, replacements, beautify)
 
     def matches(self, matcher):
         base_matched = self._base_matches(matcher)
@@ -586,15 +599,19 @@ class LLUDPMessageLogEntry(AbstractMessageLogEntry):
         for block_name in message.blocks:
             if not fnmatch.fnmatchcase(block_name, matcher.selector[1]):
                 continue
-            for block in message[block_name]:
+            for block_num, block in enumerate(message[block_name]):
                 for var_name in block.vars.keys():
                     if not fnmatch.fnmatchcase(var_name, matcher.selector[2]):
                         continue
+                    # So we know where the match happened
+                    span_key = (message.name, block_name, block_num, var_name)
                     if selector_len == 3:
+                        # We're just matching on the var existing, not having any particular value
                         if matcher.value is None:
-                            return True
+                            return span_key
                         if self._val_matches(matcher.operator, block[var_name], matcher.value):
-                            return True
+                            return span_key
+                    # Need to invoke a special unpacker
                     elif selector_len == 4:
                         try:
                             deserialized = block.deserialize_var(var_name)
@@ -608,9 +625,9 @@ class LLUDPMessageLogEntry(AbstractMessageLogEntry):
                         for key in deserialized.keys():
                             if fnmatch.fnmatchcase(str(key), matcher.selector[3]):
                                 if matcher.value is None:
-                                    return True
+                                    return span_key
                                 if self._val_matches(matcher.operator, deserialized[key], matcher.value):
-                                    return True
+                                    return span_key
 
         return False
 
